@@ -62,8 +62,7 @@ use.theme <- function(type,style="default"){
 #' @description Create itol.unit from simple input in R environment.
 #' @param data if type == "COLLAPSE", a vector of characters specifying the tips
 #' or node used for collapsing used for extracting.
-#' @param key a character specifying the theme name and prefix of data name as
-#' key information for the following reproducible workflow.
+#' @param key a character specifying the output file name for hub object.
 #' @param type a character specifying the template type used for extracting.
 #' Following choices are possible: "COLLAPSE","PRUNE","SPACING","TREE_COLORS",
 #' "DATASET_STYLE","LABELS","DATASET_TEXT","DATASET_COLORSTRIP",
@@ -151,6 +150,7 @@ use.theme <- function(type,style="default"){
 #' @import dplyr
 #' @importFrom stringr str_replace
 #' @importFrom stringr str_remove_all
+#' @importFrom stringr str_length
 #' @importFrom grDevices boxplot.stats
 #' @export
 #' @examples
@@ -252,7 +252,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(data[[potential_color]]))>3){
+        if(min(stringr::str_length(data[[potential_color]]))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
@@ -494,7 +494,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(data[[potential_color]]))>3){
+        if(min(stringr::str_length(data[[potential_color]]))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
@@ -566,7 +566,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
       if(subtype == "label"){
         potential_background_colors <- names(data)[-1]
         for(potential_background_color in potential_background_colors){
-          if(min(nchar(data[[potential_background_color]]))>3){
+          if(min(stringr::str_length(data[[potential_background_color]]))>3){
             str_1 <- unique(stringr::str_extract(data[[potential_background_color]],"^."))
             str_2 <- unique(stringr::str_extract(data[[potential_background_color]],"^..."))
             if(length(str_1)==1){
@@ -816,7 +816,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(data[[potential_color]]))>3){
+        if(min(stringr::str_length(data[[potential_color]]))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
@@ -1004,7 +1004,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(data[[potential_color]]))>3){
+        if(min(stringr::str_length(data[[potential_color]]))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
@@ -1103,6 +1103,11 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     data_left[["node"]] <- df_merge(data_left[["node"]], df_data)
     data_left[["tip"]] <- df_merge(data_left[["tip"]], df_data)
     profile$name <- key
+    common_themes$legend$title <- colname_data
+    common_themes$legend$shapes <- levels(as.factor(shape))
+    common_themes$legend$colors <- levels(as.factor(color))
+    common_themes$legend$labels <- levels(as.factor(data[[colname_data]]))
+    common_themes$legend$shape_scales <- rep(1,length(levels(as.factor(shape))))
     unit <- new("itol.unit", type = type, sep = sep, profile = profile, field = field, common_themes = common_themes, specific_themes = specific_themes, data = data_left)
   }
   if(type == "DATASET_EXTERNALSHAPE"){
@@ -1122,6 +1127,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     data_left[["node"]] <- df_merge(data_left[["node"]], data)
     data_left[["tip"]] <- df_merge(data_left[["tip"]], data)
     field$labels <- field_names
+    field_length <- length(field_names)
     if(is.null(color)){
       message("Using default color pattern: table2itol")
       color = "table2itol"
@@ -1136,6 +1142,20 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     unit <- new("itol.unit", type = type, sep = sep, profile = profile, field = field, common_themes = common_themes, specific_themes = specific_themes, data = data_left)
   }
   if(type == "DATASET_DOMAINS"){
+    shape_by = NULL
+    if(length(names(data)) == 3){
+      data <- data[order(data[[2]],data[[3]]),]
+      shape_by <- factor(pull(data[,2]),levels = unique(pull(data[,2])))
+      print(shape_by)
+      levels(shape_by) <- c("RE","HH","HV","EL","DI","TR","TL","PL","PR","PU","PD","OC","GP")[1:length(levels(shape_by))]
+      shape = shape_by
+            print(shape)
+      data <- data[,-2]
+      data[,2] <- factor(pull(data[,2]),levels = unique(pull(data[,2])))
+    }
+    if(length(names(data)) == 2){
+      data <- data.frame(data,length=rep(10,nrow(data)),start=rep(0,nrow(data)),end=rep(10,nrow(data)))
+    }
     length = NULL
     start = NULL
     end = NULL
@@ -1201,7 +1221,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(as.character(data[[potential_color]])))>3){
+        if(min(stringr::str_length(as.character(data[[potential_color]])))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
@@ -1261,7 +1281,15 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     data_left[["node"]] <- df_merge(data_left[["node"]], df_data)
     data_left[["tip"]] <- df_merge(data_left[["tip"]], df_data)
     profile$name <- key
-    common_themes$legend$shapes <- levels(as.factor(shape))
+    if(is.null(shape_by)){
+      if(length(levels(as.factor(shape)))==1){
+        common_themes$legend$shapes <- rep(levels(as.factor(shape)),length(levels(as.factor(color))))
+      }else {
+         common_themes$legend$shapes <- levels(as.factor(shape))
+      }
+    }else {
+       common_themes$legend$shapes <- shape_by[!duplicated(data[[colname_data]])]
+    }
     common_themes$legend$colors <- levels(as.factor(color))
     common_themes$legend$labels <- levels(as.factor(data[[colname_data]]))
     specific_themes$basic_plot$dataset_scale <- start %>% unique() %>% sort()
@@ -1501,7 +1529,7 @@ create_unit <- function(data,key,type,style="default",subtype=NULL,color=NULL,li
     if(is.null(color)){
       potential_colors <- names(data)[-1]
       for(potential_color in potential_colors){
-        if(min(nchar(data[[potential_color]]))>3){
+        if(min(stringr::str_length(data[[potential_color]]))>3){
           str_1 <- unique(stringr::str_extract(data[[potential_color]],"^."))
           str_2 <- unique(stringr::str_extract(data[[potential_color]],"^..."))
           if(length(str_1)==1){
